/*
 * Created on May 31, 2004
 *
 */
package archstudio.comp.cake;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import edu.uci.ics.xarchutils.ObjRef;

/**
 * Maps element <code>ObjRef</code>s to a collection of <code>InterfacePair</code>s.
 * 
 * @author Chris
 */
public class InterfaceMap
{
	/** Contains a mapping from element <code>ObjRef</code>s to a <code>Collection</code> of <code>InterfacePair</code>s. */
	protected Map elementToInterfacePairsMap; 

	/**
	 * Constructs an empty <code>InterfaceMap</code>.
	 *
	 */	
	public InterfaceMap()
	{
		elementToInterfacePairsMap = Collections.synchronizedMap(new HashMap());
	}

	/**
	 * Map an element <code>ObjRef</code> to a collection of <code>InterfacePair</code>s.
	 * 
	 * @param elementRef An <code>ObjRef</code> to an element associated with <code>InterfacePair</code>s. 
	 * @param interfacePairs A collection of <code>InterfacePair</code>s.
	 * @return The collection previously associated with the ref or 
	 * 		<code>null</code> if there was no prior mapping. 
 	 */
	public Collection mapRefToInterfacePairs(ObjRef elementRef, Collection interfacePairs)
	{
		return (Collection)elementToInterfacePairsMap.put(elementRef, interfacePairs);
	}
	
	/**
	 * Map an element <code>ObjRef</code> to a new <code>InterfacePair</code>. If a mapping already exists
	 * from this element to a collection of <code>InterfacePair</code>s then this pair is simply added to
	 * that collection. 
	 * 
	 * @param elementRef An <code>ObjRef</code> to an element associated with <code>InterfacePair</code>s.
	 * @param interfaceRef An <code>ObjRef</code> to an interface.
	 * @param interfaceID A <code>String</code> to an interface thing ID.
	 * @return The collection previously associated with the ref or 
	 * 		<code>null</code> if there was no prior mapping. 
	 */
	public Collection mapRefToInterfacePair(ObjRef elementRef, ObjRef interfaceRef, String interfaceThingID)
	{
		InterfacePair pair = new InterfacePair(interfaceRef, interfaceThingID);
		return mapRefToInterfacePair(elementRef, pair);
	}
	
	/**
	 * Map an element <code>ObjRef</code> to a new <code>InterfacePair</code>. If a mapping already exists
	 * from this element to a collection of <code>InterfacePair</code>s then this pair is simply added to
	 * that collection. 
	 * 
	 * @param elementRef An <code>ObjRef</code> to an element associated with <code>InterfacePair</code>s.
	 * @param pair An <code>InterfacePair</code> that the elementRef would map to.
	 * @return The collection previously associated with the ref or 
	 * 		<code>null</code> if there was no prior mapping. 
	 */
	public Collection mapRefToInterfacePair(ObjRef elementRef, InterfacePair pair)
	{
		Collection c = (Collection)elementToInterfacePairsMap.get(elementRef);
		if(c == null)
		{
			c = Collections.synchronizedCollection(new ArrayList()); 
		}

		c.add(pair);
		return (Collection)elementToInterfacePairsMap.put(elementRef, c);
	}
	
	/**
	 * Removes all <code>InterfacePair</code>s mapped from this element ref in this map.
	 * 
	 * @param elementRef <code>ObjRef</code> to the collection of <code>InterfacePair</code>s whose mapping is to be removed.
	 * @return The previous collection of <code>InterfacePair</code>s associated with the element ref, 
	 * 		or <code>null</code> if there was no prior mapping.
	 */
	public Collection removeInterfacePairs(ObjRef elementRef)
	{
		return (Collection)elementToInterfacePairsMap.remove(elementRef);
	}
	
	/**
	 * Removes an <code>InterfacePair</code> from the mapping.
	 * 
	 * @param elementRef <code>ObjRef</code> to the <code>InterfacePair</code> whose mapping is to be removed.
	 * @param interfaceRef The <code>ObjRef</code> to the interface ref associated with the <code>InterfacePair</code>
	 * 		to be removed.
	 * @return The previous <code>InterfacePair</code> associated with the element ref, or <code>null</code> 
	 * 		if there was no prior mapping.
	 */
	public InterfacePair removeInterfacePair(ObjRef elementRef, ObjRef interfaceRef)
	{
		Collection c = (Collection)elementToInterfacePairsMap.get(elementRef);
		if(c != null)
		{
			InterfacePair pair = null;
			for(Iterator iterator = c.iterator(); iterator.hasNext(); )
			{
				pair = (InterfacePair)iterator.next();
				if(pair.getInterfaceRef().equals(interfaceRef))
				{
					iterator.remove();
					return pair;
				}
			}
		}
		
		return null;
	}
	
	/**
	 * Removes an <code>InterfacePair</code> from the mapping.
	 * 
	 * @param elementRef <code>ObjRef</code> to the <code>InterfacePair</code> whose mapping is to be removed.
	 * @param interfaceThingID The thing ID associated with the <code>InterfacePair</code>
	 * 		to be removed.
	 * @return The previous <code>InterfacePair</code> associated with the element ref, or <code>null</code> 
	 * 		if there was no prior mapping.
	 */
	public InterfacePair removeInterfacePair(ObjRef elementRef, String interfaceThingID)
	{
		Collection c = (Collection)elementToInterfacePairsMap.get(elementRef);
		if(c != null)
		{
			InterfacePair pair = null;
			for(Iterator iterator = c.iterator(); iterator.hasNext(); )
			{
				pair = (InterfacePair)iterator.next();
				if(pair.getInterfaceThingID().equals(interfaceThingID))
				{
					iterator.remove();
					return pair;
				}
			}
		}

		return null;
	}
	
	/**
	 * Get the list of <code>InterfacePair</code>s associated with an element <code>ObjRef</code>.
	 * 
	 * @param elementRef The <code>ObjRef</code> whose associated list of <code>InterfacePair</code>s is to be returned.
	 * @return The collection to which this map maps the specified ref, or <code>null</code> if
	 * 		no mapping exists. 
	 */
	public Collection getInterfacePairs(ObjRef elementRef)
	{
		return (Collection)elementToInterfacePairsMap.get(elementRef);
	}
	
	/**
	 * Returns an interface <code>ObjRef</code> that is associated with an element <code>ObjRef</code> and has
	 * the given interface thing ID. 
	 * 
	 * @param elementRef The element <code>ObjRef</code> whose associated interface <code>ObjRef</code> is to be returned.
	 * @param interfaceThingID The thing ID of the interface to be returned.
	 * @return The interface ref mapped to by the provided element ref and thing ID.
	 */
	public ObjRef getInterfaceRef(ObjRef elementRef, String interfaceThingID)
	{
		Collection c = (Collection)elementToInterfacePairsMap.get(elementRef);
		if(c != null)
		{
			InterfacePair pair = null;
			for(Iterator iterator = c.iterator(); iterator.hasNext(); )
			{
				pair = (InterfacePair)iterator.next();
				if(pair.getInterfaceThingID().equals(interfaceThingID))
				{
					return pair.getInterfaceRef();
				}
			}			
		}
		
		return null;
	}
	
	/**
	 * Returns an interface thing ID that is associated with an element <code>ObjRef</code> and has the 
	 * given interface ref.
	 * 
	 * @param elementRef The element <code>ObjRef</code> whose associated interface <code>ObjRef</code> is to be returned.
	 * @param interfaceRef The <code>ObjRef</code> of the interface thing ID to be returned.
	 * @return The interface thing ID mapped to by the provided element ref and interface ref.
	 */
	public String getInterfaceThingID(ObjRef elementRef, ObjRef interfaceRef)
	{
		Collection c = (Collection)elementToInterfacePairsMap.get(elementRef);
		if(c != null)
		{
			InterfacePair pair = null;
			for(Iterator iterator = c.iterator(); iterator.hasNext(); )
			{
				pair = (InterfacePair)iterator.next();
				if(pair.getInterfaceRef().equals(interfaceRef))
				{
					return pair.getInterfaceThingID();
				}
			}			
		}
		 
		return null;
	}
	
	/**
	 * Returns a set view of the keys contained in this map.
	 * @return A set view of the keys contained in this map. That is, a set of element <code>ObjRef</code>s.
	 */
	public Set getKeySet()
	{
		return elementToInterfacePairsMap.keySet();
	}
	
	/**
	 * Returns the element key that has the provided interface thing ID in one of its associated 
	 * 	<code>InterfacePair</code>s 
	 * @param interfaceThingID The interface thing ID associated to check against
	 * @return The element that has the provided interface thing ID in one of its associated 
	 * 	<code>InterfacePair</code>s or <code>null</code> if no mapping exists.
	 */
	public ObjRef getElementWithInterfaceThingID(String interfaceThingID)
	{
		Set set = elementToInterfacePairsMap.keySet();
		ObjRef element = null;
		for(Iterator iterator = set.iterator(); iterator.hasNext(); )
		{
			element = (ObjRef)iterator.next();
			if(getInterfaceRef(element, interfaceThingID) != null)
			{
				return element;
			}			
		}
		
		return null;
	}
	
	/**
	 * Returns a collection of all interface refs associated with the provided element ref.
	 * 
	 * @param elementRef The element <code>ObjRef</code> whose associated interface <code>ObjRef</code>s are to be returned.
	 * @return A collection of all interface refs associated with the provided element ref, or <code>null</code> if no
	 * 		mapping exists with provided elementRef.
	 */
	public Collection getAllInterfaceRefs(ObjRef elementRef)
	{
		Collection c = (Collection)elementToInterfacePairsMap.get(elementRef);
		if(c != null)
		{						
			Collection newCollection = Collections.synchronizedCollection(new ArrayList());
			InterfacePair pair = null;
			for(Iterator iterator = c.iterator(); iterator.hasNext(); )
			{
				pair = (InterfacePair)iterator.next();
				newCollection.add(pair.getInterfaceRef()); 
			}
					
			return newCollection;
		}
		
		return null;
	}
	
	/**
	 * Returns a collection of all interface thing IDs associated with the provided element ref.
	 * 
	 * @param elementRef The element <code>ObjRef</code> whose associated interface thing IDs are to be returned.
	 * @return A collection of all interface thing IDs associated with the provided element ref, or <code>null</code> if no
	 * 		mapping exists with provided elementRef..
	 */
	public Collection getAllInterfaceThingIDs(ObjRef elementRef)
	{
		Collection c = (Collection)elementToInterfacePairsMap.get(elementRef);
		if(c != null)
		{
			Collection newCollection = Collections.synchronizedCollection(new ArrayList());
			InterfacePair pair = null;
			for(Iterator iterator = c.iterator(); iterator.hasNext(); )
			{
				pair = (InterfacePair)iterator.next();
				newCollection.add(pair.getInterfaceThingID()); 
			}
		
			return newCollection;
		}
		
		return null;			
	}
	
	/**
	 * This class describes a paring of an interface ref and the interface's thing ID. It is used as the means of
	 * storing data within an <code>InterfaceMap</code>.   
	 * 
	 * @author Chris
	 *
	 */
	public class InterfacePair
	{
		private ObjRef interfaceRef;
		private String interfaceThingID;
		
		public InterfacePair(ObjRef interfaceRef, String interfaceID)
		{
			this.interfaceRef = interfaceRef;
			this.interfaceThingID = interfaceID;
		}
		
		public ObjRef getInterfaceRef()
		{
			return interfaceRef;
		}
		
		public String getInterfaceThingID()
		{
			return interfaceThingID;
		}
	}
}
